package ru.ulstu.timeline.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import ru.ulstu.core.util.DateUtils;
import ru.ulstu.timeline.model.Event;
import ru.ulstu.timeline.model.EventDto;
import ru.ulstu.timeline.repository.EventRepository;

import java.sql.Date;
import java.time.LocalDate;
import java.time.Period;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Calendar;


public class EventScheduler {

    private final Logger log = LoggerFactory.getLogger(EventScheduler.class);
    private final EventService eventService;
    private final EventRepository eventRepository;

    public EventScheduler(EventRepository eventRepository, EventService eventService) {
        this.eventRepository = eventRepository;
        this.eventService = eventService;
    }

    @Scheduled(cron = "0 0 8/12 * 1 ?")
    public void checkPeriodEvents() {
        log.debug("EventScheduler.checkPeriodEvents started");
        for (Event event : eventRepository.findAll()) {
            if(halfOfThePariodHasPassed(event)){
                event.setCreateDate(Date.from((LocalDate.now().atStartOfDay(ZoneId.systemDefault()).toInstant())));
                event.setBeginDate(DateUtils.addDays(event.getBeginDate(), event.getPeriod().getPeriod().getDays()));
                event.setUpdateDate(null);

                eventService.create(new EventDto(
                        event.getId(),
                        event.getTitle(),
                        event.getPeriod(),
                        event.getBeginDate(),
                        event.getExecuteDate(),
                        event.getCreateDate(),
                        event.getUpdateDate(),
                        event.getDescription(),
                        new ArrayList<>()
                ));
            }
        }

        log.debug("EventScheduler.checkPeriodEvents finished");
    }

    private boolean halfOfThePariodHasPassed(Event event){
        if (Period.between(event.getBeginDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDate(),
                LocalDate.now()).getDays() <= event.getPeriod().getPeriod().getDays() / 2) {
            return true;
        }
        return false;
    }
}
